
/*
	TrayMin - PC Magazine Windows tray minimization utility

	Copyright (c) 2000 Ziff-Davis Publishing Company.  All rights reserved.
	First published in PC Magazine, US Edition  February 23, 1999.

	Written by Steven E. Sipe


	This file implements TrayMin's hook routines.
*/

#include <windows.h>
#include <stdio.h>

#define HOOK_MAIN
#include "tmzhook.h"

// Static data that's shared across all instances of the DLL
#pragma data_seg("SHARED")
static HHOOK g_hHook = NULL;
static HWND g_hwndNotify = NULL;
#pragma data_seg()

// Static data that's unique to each instance of the DLL
static HINSTANCE g_hinstDLL;

// Determine's whether the current application should be tray-minimized
BOOL ShouldMinimize(MSG *pMsg)
{
	BOOL bRet = FALSE;

	// Minimize top level windows, with an enabled minimize box, when the
	// user clicks the right mouse button on the minimize box control
	if(pMsg)
	{
		if(!IsWindowVisible(pMsg->hwnd))
			return(bRet);
		else
		{

			DWORD dwStyle = GetWindowLong(pMsg->hwnd,GWL_STYLE);

			// Check to see if the right mouse button was clicked on the 
			// minimize button of a top-level window
			if(pMsg->message == WM_NCRBUTTONUP && !GetParent(pMsg->hwnd)
					&& (dwStyle & WS_MINIMIZEBOX))
			{
				bRet = (pMsg->wParam == HTMINBUTTON);
			}
		}
	}

	return(bRet);
}

// CallWnd message hook proc
LRESULT CALLBACK GetMsgProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	LRESULT ret;
    MSG *pMsg = (MSG *) lParam;

	// Call the next hook
	ret = CallNextHookEx(g_hHook,nCode,wParam,lParam);

	// Should we process this message?
	if(nCode == HC_ACTION && wParam != PM_NOREMOVE)
	{
		// Should we handle this window?
		if(ShouldMinimize(pMsg)) 
		{
			static COPYDATASTRUCT cds;
			static TRAYDATA NewTD;

			// Valid window handle?
			if(pMsg->hwnd)
			{
				// Setup a WM_COPYDATA message to pass to the main
				// control program
				NewTD.hWnd = pMsg->hwnd;
				cds.cbData = sizeof(TRAYDATA);
				cds.lpData = (LPVOID) &NewTD;

				// Get the module's filename
				GetModuleFileName(NULL,NewTD.szFileName,MAX_PATH);

				// Send it to our main window
				SendMessage(g_hwndNotify,WM_COPYDATA,(WPARAM) pMsg->hwnd, 
									(LPARAM)&cds);
			}
		}
	}
	else return(ret);

	// Return the results
	return(TRUE);
}

// Main DLL entry point
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpvReserved)
{
	g_hinstDLL = hinstDLL;

	// Attaching new process
    if(dwReason == DLL_PROCESS_ATTACH)
    {
	}
	else if(dwReason == DLL_PROCESS_DETACH)
    {
		// Detaching process
    }

    return TRUE;
}

// Set the sender window to recieve information
void SetNotifyWindow(HWND hwnd)
{
	g_hwndNotify = hwnd;
}

// Reset the hook
void ResetHook()
{
	// Uninstall the hook
	if(g_hHook)
		UnhookWindowsHookEx(g_hHook);

	// Reset the handles
	g_hHook = NULL;
}

// Set the hook
int SetHook()
{
	// Set the hook
	g_hHook = SetWindowsHookEx(WH_GETMESSAGE,(HOOKPROC) GetMsgProc,g_hinstDLL,0);

	// Return the result
	return(g_hHook != NULL);
}
