
/*
	TrayMin - PC Magazine Windows tray minimization utility

	Copyright (c) 2000 Ziff-Davis Publishing Company.  All rights reserved.
	First published in PC Magazine, US Edition  February 23, 1999.

	Written by Steven E. Sipe


	This file implements TrayMin's tray interface routines.
*/

#include <windows.h>

extern "C"
{
#include "tray.h"
}

// Handles the "real" add/modify tray operation -- it is called by the public
// Add() and Modify() functions
static BOOL Tray_DoAddModify(HWND hWnd, int nMessageID, int nTrayID, 
						LPCTSTR lpszTitle, HICON hIcon, int nOperation)
{
	// Initialize the registration structure
	NOTIFYICONDATA notifyIcon;
	notifyIcon.cbSize = sizeof(notifyIcon);
	notifyIcon.uFlags = NIF_MESSAGE;
	notifyIcon.hWnd = hWnd;
	notifyIcon.uID = nTrayID;   
	notifyIcon.uCallbackMessage = nMessageID;

	// If setting a new icon...
	if(hIcon)
	{
		notifyIcon.uFlags |= NIF_ICON;
		notifyIcon.hIcon = hIcon;
	}

	// If setting a tooltip title...
	if(lpszTitle)
	{
		notifyIcon.uFlags |= NIF_TIP;
		strcpy(notifyIcon.szTip,lpszTitle);
	}

	// Register with the shell
	return(Shell_NotifyIcon(nOperation,&notifyIcon));
}


// Adds a new notification object to the tray
BOOL Tray_Add(HWND hWnd, int nMessageID, int nTrayID, LPCTSTR lpszTitle, HICON hIcon)
{
	BOOL bRet = FALSE;
	int nTries = 3;

	while(!bRet && nTries--)
	{
		try
		{
			bRet = Tray_DoAddModify(hWnd,nMessageID,nTrayID,lpszTitle,hIcon,NIM_ADD);
		}
		catch(...)
		{
			Sleep(30);
		}
	}

	return(bRet);
}			 

// Modifies the specified notification object
BOOL Tray_Modify(HWND hWnd, int nMessageID, int nTrayID, HICON hIcon, LPCTSTR lpszTitle)
{
	return(Tray_DoAddModify(hWnd,nMessageID,nTrayID,lpszTitle,hIcon,NIM_MODIFY));
}

// Deletes the specified notification object
BOOL Tray_Delete(HWND hWnd, int nTrayID)
{
	// Remove the notification icon from the taskbar
	NOTIFYICONDATA notifyIcon;
	notifyIcon.cbSize = sizeof(notifyIcon);
	notifyIcon.hWnd = hWnd;
	notifyIcon.uID = nTrayID;

	return(Shell_NotifyIcon(NIM_DELETE,&notifyIcon));
}

